/*
 * Created on 22.05.2006 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2509 $
 *  $Date: 2006-10-06 12:01:50 +0200 (Fr, 06 Okt 2006) $
 *  $Id: ChooseLayerAndAttributeBox.java 2509 2006-10-06 10:01:50Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.ui.panels;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import com.vividsolutions.jump.feature.AttributeType;
import com.vividsolutions.jump.feature.FeatureSchema;
import com.vividsolutions.jump.workbench.model.Layer;
import com.vividsolutions.jump.workbench.plugin.PlugInContext;

import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.DebugUserIds;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.PersonalLogger;
import de.fhOsnabrueck.jump.pirol.utilities.i18n.PirolPlugInMessages;

/**
 * a Box to choose a layer and an attribute from it's feature schema
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2006),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2509 $
 * 
 */
public class ChooseLayerAndAttributeBox extends Box implements ActionListener {

    private static final long serialVersionUID = 3579430651926642990L;
    
    protected PlugInContext context = null;
    
    protected JComboBox layersBox = null, attributesBox;
    
    protected ArrayList allowedAttributeTypes = new ArrayList();
    
    public final static Dimension defaultComponentSize = new Dimension(175,20);
    public final static Dimension maxComponentSize = new Dimension(200,20);
    
    protected PersonalLogger logger = new PersonalLogger(DebugUserIds.OLE);

    public ChooseLayerAndAttributeBox(PlugInContext context) {
        super(BoxLayout.Y_AXIS);
        this.context = context;
        
        allowedAttributeTypes.add(AttributeType.DOUBLE);
        allowedAttributeTypes.add(AttributeType.INTEGER);
        
        this.setupGui();
    }
    
    public ChooseLayerAndAttributeBox(PlugInContext context, List allowedAttributes) {
        super(BoxLayout.Y_AXIS);
        this.context = context;
        
        allowedAttributeTypes.addAll(allowedAttributes);
        
        this.setupGui();
    }
    
    protected void setupGui(){
        Box hbox = Box.createHorizontalBox();
        
        hbox.add(new JLabel(PirolPlugInMessages.getString("layer") + ": "));
        
        List layers = this.context.getLayerManager().getLayers();
        this.layersBox = new JComboBox( layers.toArray() );
        this.layersBox.addActionListener(this);
        
        this.layersBox.setMinimumSize(defaultComponentSize);
        this.layersBox.setPreferredSize(defaultComponentSize);
        this.layersBox.setMaximumSize(maxComponentSize);
        
        hbox.add(Box.createGlue());
        hbox.add(this.layersBox);
        
        this.add(hbox);
        
        hbox = Box.createHorizontalBox();
        hbox.add(new JLabel(PirolPlugInMessages.getString("attribute") + ": "));
        hbox.add(Box.createGlue());
        this.attributesBox = new JComboBox( layers.toArray() );
        this.attributesBox.setMinimumSize(defaultComponentSize);
        this.attributesBox.setPreferredSize(defaultComponentSize);
        this.attributesBox.setMaximumSize(maxComponentSize);
        hbox.add(this.attributesBox);
        this.add(hbox);
        
        this.add(Box.createGlue());
        
        this.fillInCurrentAttributes();
    }
    
    protected void fillInCurrentAttributes(){
        Layer layer = (Layer)this.layersBox.getSelectedItem();
        
        this.attributesBox.removeAllItems();
        
        FeatureSchema fs = layer.getFeatureCollectionWrapper().getUltimateWrappee().getFeatureSchema();
        
        for (int i=0; i<fs.getAttributeCount(); i++){
            if (allowedAttributeTypes.contains(fs.getAttributeType(i))){
                this.attributesBox.addItem(fs.getAttributeName(i));
            }
        }
    }
    
    public void setEnabled(boolean enable) {
        this.layersBox.setEnabled(enable);
        this.fillInCurrentAttributes();
        this.attributesBox.setEnabled(enable);
    }
    
    

    public void setVisible(boolean visible) {
        super.setVisible(visible);
        this.layersBox.setVisible(visible);
        this.attributesBox.setVisible(visible);
    }

    /**
     *@param arg0
     */
    public void actionPerformed(ActionEvent arg0) {
        // layer selection changed
        this.fillInCurrentAttributes();
    }

    public String getAttributeName() {
        return (String)attributesBox.getSelectedItem();
    }

    public Layer getSelectedLayer() {
        return (Layer)layersBox.getSelectedItem();
    }    

}
